import 'package:flutter/material.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:flutter/src/widgets/placeholder.dart';
import 'package:loading_overlay/loading_overlay.dart';
import 'package:storebounty_checkout/model/locationdatamodel.dart';
import 'package:storebounty_checkout/src/entrypoint.dart';
import 'package:storebounty_checkout/storebounty_checkout_v1.dart';
import 'package:storebounty_checkout/widget/custombutton.dart';
import 'package:storebounty_checkout/widget/customdropdown2.dart';
import 'package:storebounty_checkout/widget/custominputtextfield.dart';
import 'package:storebounty_checkout/widget/exploreappbar.dart';
import 'package:storebounty_checkout/widget/styles/colors.dart';
import 'package:storebounty_init/storebountyinit.dart';

class DeliveryForm extends StatefulWidget {
  static const pageId = "delivery_form";
  const DeliveryForm({super.key});

  @override
  State<DeliveryForm> createState() => _DeliveryFormState();
}

class _DeliveryFormState extends State<DeliveryForm> {
  var _formKey = GlobalKey<FormState>();
  bool isSubmiting = false;
  Color? defaultColor;
  TextEditingController firstnameCtx = TextEditingController();
  TextEditingController lastnameCtx = TextEditingController();
  TextEditingController emailCtx = TextEditingController();
  TextEditingController phoneCtx = TextEditingController();
  TextEditingController addressCtx = TextEditingController();
  List<Country> countriesList = [];
  List<Region> stateList = [];
  List<LocalArea> localAreaList = [];

  Country? selectedCountryComplete;
  Region? selectedStateComplete;
  List<Map<String, String>> countryToDisplay = [];
  List<Map<String, String>> stateListMap = [];
  List<Map<String, String>> localAreaListMap = [];
  var selectedCountry = "";
  String selectedState = "";
  String selectedCity = "";

  @override
  void initState() {
    getColor();
    getStore();
    super.initState();
  }

  getColor() async {
    var colorString = await StoreBountyInit.getStoreColor() ?? "#000000";
    defaultColor = Color(int.parse("0xff" + colorString.substring(1)));
    var currencySymbol = await StoreBountyInit.getCurrencySymbol();
    // currency = currencySymbol!;
    setState(() {});
  }

  getStore() async {

    setState(() {
      countryToDisplay = [{"name":"Select...", "value":""}];
      stateListMap = [{"name":"Select...", "value":""}];
      localAreaListMap = [{"name":"Select...", "value":""}];
      countriesList = [];
    });

    final response = await StorebountyCheckout.getLocationData();

    final data = locationDataModelFromMap(response.body);

    var countries = data.data!.countries;
    countriesList.addAll(countries ?? []);

    for(var i = 0; i < countries!.length; i++){
      if(countries[i].name?.toLowerCase() == "nigeria"){
        print("object");
        selectedCountryComplete = countries[i];
        countryToDisplay.add({"name":"${countries[i].name}", "value":"${countries[i].code}"});
      }
    }
    setState(() {});
  }

  getStatesForSelectedCountry(value){
    selectedState = "";
    // stateListMap = [{"name":"Select...", "value":""}];
    stateList.clear();
    setState(() {

    });

      if(selectedCountryComplete != null){
        print(value);
        stateList.addAll(selectedCountryComplete?.regions ?? []);
      }

    for(var i = 0; i < stateList.length; i++){
        stateListMap.add({"name":"${stateList[i].name}", "value":"${stateList[i].code}"});
    }

    print(stateListMap);

    setState(() {});
  }

  getLocalAreaData(value){

    for(var i = 0; i < stateList.length; i++){
      if(stateList[i].code == value){
        selectedStateComplete = stateList[i];
      }
    }

    for(var i = 0; i < selectedStateComplete!.localAreas!.length; i++){
      var data = selectedStateComplete!.localAreas![i];
      localAreaListMap.add({"name":"${data.name}", "value":"${data.code}"});
    }

    setState(() {

    });

  }

  createAddress()async{

    try{

      var data =
      {
        "id": null,
        "first_name": firstnameCtx.text.trim(),
        "last_name": lastnameCtx.text.trim(),
        "phone": phoneCtx.text.trim(),
        "email": emailCtx.text.trim(),
        "address": addressCtx.text.trim(),
        "city": selectedCity,
        "local_area_code": selectedCity,
        "post_code": "",
        "landmark": "",
        "country_code": selectedCountry,
        "region_code": selectedState,
        "is_default":true,
        "google_place_id": "",
        "google_map_search_payload": ""
      };

      final response = await StorebountyCheckout.createAddress(data);
      await StorebountyCheckout.getAddresses();


    }catch(e){

    }

  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () {
        FocusScope.of(context).unfocus();
      },
      child: LoadingOverlay(
        isLoading: isSubmiting,
        child: Scaffold(
          appBar: ExploreAppBar(
            title: "${countriesList.length}",
            onClick: () {
              Navigator.pop(context);
            },
            showLeftIcon: false,
          ),
          body: Container(
            child: SingleChildScrollView(
              child: Form(
                key: _formKey,
                child: Column(
                  children: [
                    SizedBox(
                      height: 20,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 24),
                      child: CustomTextFormField(
                        hintText: "First name",
                        labelText: 'First Name',
                        controller: firstnameCtx,
                        validator: (value) {
                          if (value == "") {
                            return "required";
                          }
                        },
                      ),
                    ),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 24),
                      child: CustomTextFormField(
                          hintText: "Last name",
                          labelText: 'Last Name',
                          controller: lastnameCtx,
                          validator: (value) {
                            if (value == "") {
                              return "required";
                            }
                          }),
                    ),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 24),
                      child: CustomTextFormField(
                          hintText: "Enter email",
                          labelText: 'Email',
                          controller: emailCtx,
                          validator: (value) {
                            if (value == "") {
                              return "required";
                            }
                          }),
                    ),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 24),
                      child: CustomTextFormField(
                          hintText: "Enter phone",
                          labelText: 'Phone',
                          controller: phoneCtx,
                          validator: (value) {
                            if (value == "") {
                              return "required";
                            }
                          }),
                    ),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                        margin: EdgeInsets.symmetric(horizontal: 24),
                        child: CustomDropDown(
                            options: countryToDisplay,
                            hintText: 'Select...',
                            labelText: 'Country',
                            value: selectedCountry,
                            onChanged: (value) {
                              selectedCountry = value;
                              selectedState = "";
                              getStatesForSelectedCountry(value);
                            })),
                    SizedBox(
                      height: 5,
                    ),
                     Container(
                        margin: EdgeInsets.symmetric(horizontal: 24),
                        child: CustomDropDown(
                            options: stateListMap,
                            hintText: 'Select...',
                            labelText: 'State',
                            value: selectedState,
                            onChanged: (value) {
                              selectedState = value;
                              getLocalAreaData(value);
                            })),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                        margin: EdgeInsets.symmetric(horizontal: 24),
                        child: CustomDropDown(
                            options: localAreaListMap,
                            hintText: 'Select...',
                            labelText: 'City',
                            value: selectedCity,
                            onChanged: (value) {
                              print(value);
                              selectedCity = value;
                              setState(() {});
                            })),
                    SizedBox(
                      height: 5,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 24),
                      child: CustomTextFormField(
                          hintText: "Enter Address",
                          labelText: 'Address',
                          controller: addressCtx,
                          surfixWidget: InkWell(
                              onTap: () {
                                // Navigator.pushNamed(
                                //     context, AddAddressScreen.pageId);
                              },
                              child: Icon(
                                Icons.location_on,
                                color: defaultColor ?? AppColors.black,
                              )),
                          validator: (value) {
                            if (value == "") {
                              return "required";
                            }
                          }),
                    ),
                    SizedBox(
                      height: 46,
                    ),
                    Container(
                        margin: EdgeInsets.symmetric(horizontal: 24),
                        child: CustomButton(
                            text: 'Save Location',
                            color: defaultColor ?? AppColors.black,
                            onPressed: () {
                              if (!_formKey.currentState!.validate()) {
                                return;
                              }
                              setState(() {
                                isSubmiting = true;
                              });
                              createAddress();
                              setState(() {
                                isSubmiting = false;
                              });
                            })),
                    SizedBox(
                      height: 20,
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
