import 'package:flutter/material.dart';
import 'package:flutter_paystack/flutter_paystack.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:storebounty_auth/storebounty_auth_v1.dart';
import 'package:storebounty_cart_manager/model/cartmodel.dart';
import 'package:storebounty_cart_manager/storebounty_cart_manager.dart';
import 'package:storebounty_checkout/common/constants/constant.dart';
import 'package:storebounty_checkout/common/extentions.dart';
import 'package:storebounty_checkout/model/addressmodel.dart';
import 'package:storebounty_checkout/model/customermodel.dart';
import 'package:storebounty_checkout/model/locationdatamodel.dart';
import 'package:storebounty_checkout/model/ordercreatedmodel.dart';
import 'package:storebounty_checkout/model/paymentmodel.dart';
import 'package:storebounty_checkout/src/address/delivery_form.dart';
import 'package:storebounty_checkout/storebounty_checkout_v1.dart';
import 'package:storebounty_checkout/widget/alert/alertclass.dart';
import 'package:storebounty_checkout/widget/custombutton.dart';
import 'package:storebounty_checkout/widget/customdropdown.dart';
import 'package:storebounty_checkout/widget/custominputtextfield.dart';
import 'package:storebounty_checkout/widget/styles/colors.dart';
import 'package:storebounty_checkout/widget/styles/textstles.dart';
import 'package:storebounty_init/model/storedetailsmodel.dart';
import 'package:storebounty_init/storebountyinit.dart';

class StoreBountyCheckoutV2 extends StatefulWidget {
  final VoidCallback onCompleted;
  const StoreBountyCheckoutV2({Key? key, required this.onCompleted}) : super(key: key);

  @override
  State<StoreBountyCheckoutV2> createState() => _StoreBountyCheckoutV2State();
}

class _StoreBountyCheckoutV2State extends State<StoreBountyCheckoutV2> {
  List<AddressModelMain> addressList = [];
  String deliveryMethod = "doorstep";
  Color? defaultColor;
  List<dynamic> images = [];
  String currency = "NGN";
  PickupLocation? selectedPickup;
  ShippingService? selectedShipping;
  PaymentGateway? selectedPaymentGameWay;
  String paymentMode = "online";
  AddressModelMain? selectedAddress;
  String isLogisticAvailable = "none";
  FluffyRegion? selectedShippingRegion;
  double totalAmountMain = 0.0;
  bool allDigital = false;
  var deliveryFee = 0.0;
  OrderCreatedModel? orderCreatedModel;
  bool isCreatingOrder = false;

  final plugin = PaystackPlugin();

  @override
  void initState() {
    getAddresses();
    getColor();
    checkProductType();
    initPaystack();
    super.initState();
  }

  getAddresses() async {
    try {
      setState(() {
        isCreatingOrder = true;
      });
      addressList.clear();

      final response = await StorebountyCheckout.getAddresses();

      var data = addressModelFromMap(response.body);

      setState(() {
        addressList.addAll(data.data!);
        isCreatingOrder = false;
      });

    } catch (e) {
      print(e);
      setState(() {
        isCreatingOrder = false;
      });
    }
  }

  getColor() async {
    var colorString = await StoreBountyInit.getStoreColor() ?? "#000000";
    defaultColor = Color(int.parse("0xff" + colorString.substring(1)));
    var currencySymbol = await StoreBountyInit.getCurrencySymbol();
    currency = currencySymbol!;
    setState(() {});
  }

  processAddress(ShippingService value) {
    if (deliveryMethod == "doorstep") {
      setState(() {
        isLogisticAvailable = "notAvailable";
      });

      Region? gottenRegion;

      if (value.countries!.isNotEmpty) {
        for (var i = 0; i < value.countries![0].regions!.length; i++) {
          var singleRegion = value.countries![0].regions![i];
          print("current region ${singleRegion.code}");
          if (selectedAddress!.regionCode == singleRegion.code) {
            print("current region main ${singleRegion.name}");
            if (singleRegion.active == false) {
              setState(() {
                isLogisticAvailable = "notAvailable";
              });
              return;
            } else {
              setState(() {
                isLogisticAvailable = "available";
                selectedShippingRegion = singleRegion;
              });
            }
            return;
          }
        }

        if (gottenRegion != null) {
          for (var i = 0; i < gottenRegion.localAreas!.length; i++) {
            var lga = gottenRegion.localAreas![i];
            if (selectedAddress!.localAreaCode == lga.code) {}
          }
        }
      }
    }
  }

  checkProductType() async {
    var check = await StoreBountyCartManager.calculateTotals();

    setState(() {
      allDigital = check['allDigital'];
    });
  }

  processPickup(PickupLocation pickupLocation) {
    // print(pickupLocation.);
  }

  initPaystack() async {
    final store = await StoreBountyInit.getStore();

    for (var i = 0; i < store!.data!.info!.paymentGateways!.length; i++) {
      var singlePayment = store.data!.info!.paymentGateways![i];
      if (singlePayment.name == "paystack") {
        plugin.initialize(publicKey: singlePayment.publicKey!);
      }
    }
  }

  createOrder() async {
    try {
      setState(() {
        isCreatingOrder = true;
      });
      var cartItems = await StoreBountyCartManager.getCartItems();
      var customer = await StorebountyAuth.getUser();

      var items = [];

      for (var i = 0; i < cartItems.length; i++) {
        var data = {
          "product_id": cartItems[i].productId,
          "quantity": cartItems[i].quantity,
          "variants": cartItems[i].variants
        };

        items.add(data);
      }

      var logisticMain = [];

      if (selectedShipping != null) {
        ShippingService logisticService = selectedShipping!;
        logisticMain = logisticService.toMap().remove("countries");
      }

      var mainDataCreateOrder = {
        "discount_code": "",
        "pay_on_delivery": paymentMode == "online" ? false : true,
        "items": items,
        "customer": customer,
        "channel": "mobile",
        "delivery_address": {
          "first_name": selectedAddress?.firstName,
          "last_name": selectedAddress?.lastName,
          "email": selectedAddress?.email,
          "phone": selectedAddress?.phone,
          "address": selectedAddress?.address,
          "city": selectedAddress?.city,
          "state": selectedAddress?.regionCode,
          "post_code": selectedAddress?.postCode,
          "landmark": selectedAddress?.landmark
        },
        "pickup_address":
            selectedPickup == null ? null : selectedPickup!.toMap(),
        "delivery_charge": deliveryFee,
        "gateway_charges": processPaymentOptions(),
        "delivery_option": deliveryMethod,
        "delivery_service": logisticMain.length > 0 ? logisticMain[0] : {},
        "form_fields": {
          "message": "Message or more information for the delivery man."
        },
        "payment_option": "online",
        "payment_gateway": selectedPaymentGameWay?.name,
      };

      if (deliveryMethod == "pickup") {
        mainDataCreateOrder.remove("delivery_address");
      } else {
        mainDataCreateOrder.remove("pickup_address");
      }

      print(mainDataCreateOrder);

      final response =
          await StorebountyCheckout.createOrder(mainDataCreateOrder);

      print(response);

      orderCreatedModel = orderCreatedModelFromMap(response.body);

      getPaymentRef();
    } catch (e) {
      setState(() {
        isCreatingOrder = false;
      });
      print(e);
    }
  }

  getPaymentRef() async {
    try {
      var data = {
        "order_id": orderCreatedModel!.data!.orderId,
        "gateway": selectedPaymentGameWay!.name,
        "generate_link": true
      };

      final response = await StorebountyCheckout.getReferenceNo(data);

      var responseDecode = paymentModelFromMap(response.body);

      print(responseDecode.data!.amount);

      var amount = (totalAmountMain) * 100;

      Charge charge = Charge()
        ..amount = amount.toInt()
        ..reference = responseDecode.data!.reference
        // ..accessCode = themeStore.paymentGatewayData!.reference
        ..email = "${responseDecode.data!.email}";
      CheckoutResponse responsePayment = await plugin.checkout(
        context,
        method: CheckoutMethod.card, // Defaults to CheckoutMethod.selectable
        charge: charge,
      );

      var payload = {'reference': responseDecode.data!.reference};

      if (responsePayment.message.contains('Transaction terminated')) {
      } else {
        // prodNotif.isCreatingOrder = true;
        // prodNotif.notifyListeners();
        // await ref
        //     .read(productChangeNotifier)
        //     .verifyTransaction(context, payload);
        // prodNotif.isCreatingOrder = false;
        // prodNotif.notifyListeners();
        final response = await StorebountyCheckout.verifyTransaction(payload);
        await StoreBountyCartManager.clearCartItems();

        // Future.delayed(Duration.zero, () async {
        //   Navigator.of(context).pushAndRemoveUntil(
        //       MaterialPageRoute(builder: (context) => ThemeEntry()),
        //       (Route<dynamic> route) => false);
        // }).then((value) {
        //   AlertClass.success(context, "Order placed successfully");
        // });
      }

      setState(() {
        isCreatingOrder = false;
        widget.onCompleted();
      });
    } catch (e) {
      setState(() {
        isCreatingOrder = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Positioned.fill(
          child: Scaffold(
            backgroundColor: Colors.white,
            appBar: AppBar(
              elevation: 0,
              leading: InkWell(
                onTap: () {
                  Navigator.of(context).pop();
                },
                child: Icon(
                  Icons.arrow_back_ios,
                  color: Color(0xff1A2B3D),
                ),
              ),
              title: Text(
                "Checkout",
                style: TextStyle(fontSize: 16, color: Color(0xff1A2B3D)),
              ),
              backgroundColor: Colors.white,
            ),
            body: Container(
              child: SingleChildScrollView(
                child: Column(
                  children: [
                    Visibility(
                      visible: allDigital == false,
                      child: Container(
                        child: Column(
                          children: [
                            Container(
                              margin: EdgeInsets.symmetric(horizontal: 20),
                              child: Row(
                                children: [
                                  Expanded(
                                    child: InkWell(
                                      onTap: () {
                                        setState(() {
                                          deliveryMethod = "doorstep";
                                        });
                                      },
                                      child: Container(
                                        child: Column(
                                          children: [
                                            Text(
                                              "Delivery",
                                              style: Styles.p3(fontSize: 14),
                                            ),
                                            SizedBox(
                                              height: 10,
                                            ),
                                            Container(
                                              height: 2,
                                              decoration: BoxDecoration(
                                                color:
                                                    deliveryMethod == "doorstep"
                                                        ? defaultColor
                                                        : Color(0xffDADADA),
                                              ),
                                            )
                                          ],
                                        ),
                                      ),
                                    ),
                                  ),
                                  Expanded(
                                    child: InkWell(
                                      onTap: () {
                                        setState(() {
                                          deliveryMethod = "pickup";
                                        });
                                      },
                                      child: Container(
                                        child: Column(
                                          children: [
                                            Text(
                                              "Pickup",
                                              style: Styles.p3(fontSize: 14),
                                            ),
                                            SizedBox(
                                              height: 10,
                                            ),
                                            Container(
                                              height: 2,
                                              decoration: BoxDecoration(
                                                color: deliveryMethod == "pickup"
                                                    ? defaultColor
                                                    : Color(0xffDADADA),
                                              ),
                                            )
                                          ],
                                        ),
                                      ),
                                    ),
                                  )
                                ],
                              ),
                            ),
                            SizedBox(
                              height: 30,
                            ),
                            Visibility(
                                visible: deliveryMethod == "doorstep",
                                child: delivery(context)),
                            Visibility(
                                visible: deliveryMethod == "pickup",
                                child: pickUp(context))
                          ],
                        ),
                      ),
                    ),
                    SizedBox(height: 8,),
                    Row(
                      children: [
                        Container(
                          margin:
                          EdgeInsets.symmetric(horizontal: 20, vertical: 8),
                          child: Text(
                            "Payment Method",
                            style: Styles.p0(
                                fontSize: 15, fontWeight: FontWeight.w500),
                          ),
                        ),
                      ],
                    ),
                    SizedBox(
                      height: 8,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 20),
                      child: Row(
                        children: [
                          Container(
                            width: 35,
                            child: Radio(
                                value: paymentMode,
                                groupValue: "online",
                                activeColor: defaultColor ?? AppColors.black,
                                onChanged: (value) {
                                  setState(() {
                                    paymentMode = "online";
                                  });
                                },
                                materialTapTargetSize:
                                MaterialTapTargetSize.shrinkWrap),
                          ),
                          Expanded(
                            child: Container(
                              height: 48,
                              child: Row(
                                children: [
                                  SizedBox(
                                    width: 20,
                                  ),
                                  SvgPicture.asset(
                                      "assets/icons/payonline.svg"),
                                  SizedBox(
                                    width: 5,
                                  ),
                                  Text("Pay online"),
                                ],
                              ),
                              decoration: BoxDecoration(
                                  color: Color(0xffF5F5F5),
                                  borderRadius: BorderRadius.circular(4)),
                            ),
                          )
                        ],
                      ),
                    ),
                    Visibility(
                      visible: paymentMode == "online",
                      child: Column(
                        children: [
                          SizedBox(
                            height: 8,
                          ),
                          Container(
                            margin: EdgeInsets.symmetric(horizontal: 25),
                            child: Row(
                              children: [
                                SizedBox(
                                  width: 35,
                                ),
                                Expanded(child: myWidget(context)),
                              ],
                            ),
                          ),
                        ],
                      ),
                    ),
                    Visibility(
                      visible: allDigital == false,
                      child: Column(
                        children: [
                          SizedBox(
                            height: 8,
                          ),
                          Container(
                            margin: EdgeInsets.symmetric(horizontal: 20),
                            child: Row(
                              children: [
                                Container(
                                  width: 35,
                                  child: Radio(
                                      value: paymentMode,
                                      activeColor:
                                      defaultColor ?? AppColors.black,
                                      groupValue: "offline",
                                      onChanged: (value) {
                                        setState(() {
                                          paymentMode = "offline";
                                          selectedPaymentGameWay = null;
                                        });
                                      },
                                      materialTapTargetSize:
                                      MaterialTapTargetSize.shrinkWrap),
                                ),
                                Expanded(
                                  child: Container(
                                    height: 48,
                                    child: Row(
                                      children: [
                                        SizedBox(
                                          width: 20,
                                        ),
                                        SvgPicture.asset(
                                            "assets/icons/payondelivery.svg"),
                                        SizedBox(
                                          width: 5,
                                        ),
                                        Text("Pay on Delivery"),
                                      ],
                                    ),
                                    decoration: BoxDecoration(
                                        color: Color(0xffF5F5F5),
                                        borderRadius: BorderRadius.circular(4)),
                                  ),
                                )
                              ],
                            ),
                          ),
                          SizedBox(
                            height: 24,
                          ),
                        ],
                      ),
                    ),
                    Row(
                      children: [
                        Container(
                          margin:
                              EdgeInsets.symmetric(horizontal: 20, vertical: 8),
                          child: Text(
                            "Items",
                            style: Styles.p0(
                                fontSize: 15, fontWeight: FontWeight.w500),
                          ),
                        ),
                      ],
                    ),
                    Container(
                      child: FutureBuilder<List<CartModel>>(
                          future: StoreBountyCartManager.getCartItems(),
                          builder: (context,
                              AsyncSnapshot<List<CartModel>> snapshot) {
                            return ListView.builder(
                                itemCount: snapshot.data!.length,
                                shrinkWrap: true,
                                padding: EdgeInsets.zero,
                                physics: const NeverScrollableScrollPhysics(),
                                itemBuilder: (context, index) {
                                  final data = snapshot.data![index];
                                  return Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Column(
                                      children: [
                                        Row(
                                          children: [
                                            Expanded(
                                              child: Column(
                                                crossAxisAlignment:
                                                    CrossAxisAlignment.start,
                                                children: [
                                                  Row(
                                                    children: [
                                                      Text(
                                                        "${data.quantity} * ",
                                                        style: Styles.p3(
                                                            fontSize: 14,
                                                            fontWeight:
                                                                FontWeight
                                                                    .w400),
                                                      ),
                                                      Text(
                                                        "${data.title}",
                                                        style: Styles.p3(
                                                            fontSize: 14,
                                                            fontWeight:
                                                                FontWeight
                                                                    .w400),
                                                      ),
                                                    ],
                                                  ),
                                                  Text(
                                                      "${validateVariate(data)}",
                                                      style: Styles.p3(
                                                          fontSize: 10,
                                                          fontWeight:
                                                              FontWeight.w400,
                                                          color: Color(
                                                              0xffA4AAAD)))
                                                ],
                                              ),
                                            ),
                                            Column(
                                              mainAxisAlignment:
                                                  MainAxisAlignment.end,
                                              children: [
                                                Text(
                                                    "${data.priceNoVat.toString().moneyFormat}",
                                                    style: Styles.p3(
                                                        fontSize: 14,
                                                        fontWeight:
                                                            FontWeight.w400))
                                              ],
                                            )
                                          ],
                                        ),
                                        Divider()
                                      ],
                                    ),
                                  );
                                });
                          }),
                    ),

                    SizedBox(
                      height: 8,
                    ),
                    Row(
                      children: [
                        Container(
                          margin:
                              EdgeInsets.symmetric(horizontal: 20, vertical: 8),
                          child: Text(
                            "Payment Summary",
                            style: Styles.p0(
                                fontSize: 15, fontWeight: FontWeight.w500),
                          ),
                        ),
                      ],
                    ),
                    SizedBox(
                      height: 8,
                    ),
                    Container(
                      decoration: BoxDecoration(
                        color: Color(0xffF5F5F5),
                        borderRadius: BorderRadius.circular(4),
                      ),
                      padding: EdgeInsets.symmetric(vertical: 10),
                      margin: EdgeInsets.symmetric(horizontal: 20),
                      child: FutureBuilder<dynamic>(
                          future: StoreBountyCartManager.calculateTotals(),
                          builder: (context, AsyncSnapshot<dynamic> snapshot) {
                            if (snapshot.hasData) {
                              return Column(
                                children: [
                                  SizedBox(
                                    height: 13,
                                  ),
                                  Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: [
                                        Text("Sub total",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500)),
                                        Text(
                                            "${snapshot.data['currency']}${snapshot.data['subtotal'].toString().moneyFormat}",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500))
                                      ],
                                    ),
                                  ),
                                  SizedBox(
                                    height: 11,
                                  ),
                                  Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: [
                                        Text("VAT",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500)),
                                        Text(
                                            "${snapshot.data['currency']}${snapshot.data['vat'].toString().moneyFormat}",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500))
                                      ],
                                    ),
                                  ),
                                  SizedBox(
                                    height: 11,
                                  ),
                                  Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: [
                                        Text("Delivery Fee",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500)),
                                        Text(
                                            "${snapshot.data['currency']}${processDeliveryFee(snapshot)}",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500))
                                      ],
                                    ),
                                  ),
                                  SizedBox(
                                    height: 11,
                                  ),
                                  Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: [
                                        const Text("Gateway Fee",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500)),
                                        Text(
                                            "${snapshot.data['currency']}${processPaymentOptions().toStringAsFixed(2)}",
                                            style: TextStyle(
                                                fontSize: 12,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500))
                                      ],
                                    ),
                                  ),
                                  SizedBox(
                                    height: 11,
                                  ),
                                  Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: [
                                        Text("Discount (EMPEROR)",
                                            style: TextStyle(
                                                fontSize: 10,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500)),
                                        Text(
                                            "-${snapshot.data['currency']}${snapshot.data['discount'].toString().moneyFormat}",
                                            style: TextStyle(
                                                fontSize: 10,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500))
                                      ],
                                    ),
                                  ),
                                  SizedBox(
                                    height: 11,
                                  ),
                                  Container(
                                    margin:
                                        EdgeInsets.symmetric(horizontal: 20),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: [
                                        Text("Total",
                                            style: TextStyle(
                                                fontSize: 16,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500)),
                                        Text(
                                            "${snapshot.data['currency']}${(totalAmount(snapshot)).toString().moneyFormat}",
                                            style: TextStyle(
                                                fontSize: 16,
                                                color: Color(0xff1A2B3D),
                                                fontWeight: FontWeight.w500))
                                      ],
                                    ),
                                  )
                                ],
                              );
                            } else if (snapshot.hasError) {
                              return Text("Error");
                            } else {
                              return Text("Loading...");
                            }
                          }),
                    ),
                    SizedBox(
                      height: 24,
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 20, vertical: 8),
                      child: CustomButton(
                        text: 'Place Order',
                        onPressed: () {
                          createOrder();
                        },
                        color: defaultColor ?? AppColors.black,
                      ),
                    ),
                    Container(
                      margin: EdgeInsets.symmetric(horizontal: 20, vertical: 8),
                      child: CustomButton(
                        text: 'Cancel',
                        onPressed: () {},
                        textColor: Colors.red,
                        color: Color(0xffffffff),
                      ),
                    ),
                    SizedBox(
                      height: 20,
                    )
                  ],
                ),
              ),
            ),
          ),
        ),
        Visibility(
          visible: isCreatingOrder,
          child: Positioned.fill(
              child: Container(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [CircularProgressIndicator(color: defaultColor)],
            ),
            color: defaultColor?.withOpacity(0.2),
          )),
        )
      ],
    );
  }

  Widget _buildDropdownItem(AddressModelMain item) {
    return Container(
      width: MediaQuery.of(context).size.width / 1.5,
      child: Row(
        children: [
          Icon(Icons.location_on_outlined),
          SizedBox(width: 8),
          Expanded(child: Text(item.address!)),
        ],
      ),
    );
  }

  validateVariate(CartModel value) {
    if (value.variants!.isEmpty) {
      return "-";
    }

    var options = [];

    for (var i = 0; i < value.variants![0].options!.length; i++) {
      var singleOption = value.variants![0].options![i];
      options.add("${singleOption.name} : ${singleOption.option}");
    }

    String optionsMain = options.join(', ');

    return optionsMain;
  }

  Widget myWidget(BuildContext context) {
    return MediaQuery.removePadding(
      context: context,
      removeTop: true,
      child: FutureBuilder<List<PaymentGateway>>(
          future: StoreBountyInit.getPaymentGate(),
          builder: (context, AsyncSnapshot<List<PaymentGateway>> snapshot) {
            if (snapshot.hasData) {
              return GridView.builder(
                  gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                      crossAxisCount: 2,
                      childAspectRatio: MediaQuery.of(context).size.width /
                          (MediaQuery.of(context).size.height / 7)),
                  itemCount: snapshot.data!.length,
                  padding: EdgeInsets.zero,
                  physics: const NeverScrollableScrollPhysics(),
                  shrinkWrap: true,
                  itemBuilder: (BuildContext context, int index) {
                    var data = snapshot.data![index];
                    return Container(
                      child: Row(
                        children: [
                          Container(
                              width: 30,
                              child: Radio(
                                  value: data.name,
                                  activeColor: defaultColor ?? AppColors.black,
                                  groupValue: selectedPaymentGameWay?.name,
                                  onChanged: (value) {
                                    setState(() {
                                      selectedPaymentGameWay = data;
                                    });
                                  })),
                          Expanded(
                            child: Container(
                              height: 40,
                              decoration: BoxDecoration(
                                  borderRadius: BorderRadius.circular(4),
                                  border: Border.all(
                                      width: 0.6, color: Color(0xffE0E0E0))),
                              child: Padding(
                                padding: const EdgeInsets.all(8.0),
                                child: Center(
                                    child: Image.network(
                                  "${data.logo}",
                                  fit: BoxFit.fitHeight,
                                )),
                              ),
                            ),
                          )
                        ],
                      ),
                    );
                  });
            } else if (snapshot.hasError) {
              return Text("Error");
            } else {
              return Text("Loading");
            }
          }),
    );
  }

  Widget delivery(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Visibility(
          visible: deliveryMethod == "doorstep",
          child: Column(
            children: [
              GestureDetector(
                onTap: () {
                  Navigator.of(context).push(
                      MaterialPageRoute(builder: (context) => DeliveryForm()));
                },
                child: Container(
                  margin: EdgeInsets.symmetric(horizontal: 25),
                  child: Row(
                    children: [
                      Text("Delivery Address"),
                      Spacer(),
                      Icon(Icons.add),
                      Text("Add New Address")
                    ],
                  ),
                ),
              ),
              SizedBox(
                height: 5,
              ),
              Container(
                margin: EdgeInsets.symmetric(horizontal: 20),
                decoration: BoxDecoration(
                    // border: Border.all(width: 1, color: Color(0xffE6E6E6)),
                    borderRadius: BorderRadius.circular(4)),
                child: Container(
                  margin: EdgeInsets.symmetric(horizontal: 10),
                  child: Column(
                    children: [
                      Container(
                        decoration: BoxDecoration(
                            border:
                                Border.all(width: 1, color: Color(0xffE6E6E6)),
                            borderRadius: BorderRadius.circular(4)),
                        child: Padding(
                          padding: const EdgeInsets.all(4.0),
                          child: Row(
                            children: [
                              Expanded(
                                  child: DropdownButtonWithIcon(
                                items: addressList,
                                itemBuilder: (item) => _buildDropdownItem(item),
                                onChanged: (value) {
                                  print("drop down ${value!.regionCode}");
                                  setState(() {
                                    selectedAddress = value;
                                    isLogisticAvailable = "none";
                                    selectedShipping = null;
                                    selectedShippingRegion = null;
                                  });
                                },
                                hintWidget: Row(
                                  children: [
                                    Icon(Icons.location_on_outlined),
                                    SizedBox(
                                      width: 10,
                                    ),
                                    Text("Delivery Address"),
                                  ],
                                ),
                              )),
                            ],
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
              SizedBox(
                height: 20,
              ),
              Container(
                margin: EdgeInsets.symmetric(horizontal: 25),
                child: Column(
                  children: [
                    Row(
                      children: [
                        Text("Logistics Company"),
                      ],
                    ),
                    Container(
                      child: FutureBuilder<List<ShippingService>>(
                          future: StoreBountyInit.getLogistic(),
                          builder: (context,
                              AsyncSnapshot<List<ShippingService>> snapshot) {
                            if (snapshot.hasData) {
                              return ListView.builder(
                                  itemCount: snapshot.data!.length,
                                  shrinkWrap: true,
                                  padding: EdgeInsets.zero,
                                  physics: const NeverScrollableScrollPhysics(),
                                  itemBuilder: (context, index) {
                                    final data = snapshot.data![index];
                                    return Container(
                                      margin:
                                          EdgeInsets.symmetric(vertical: 10),
                                      child: Row(
                                        children: [
                                          Radio(
                                              value: data.id,
                                              activeColor: defaultColor ??
                                                  AppColors.black,
                                              groupValue: selectedShipping?.id,
                                              onChanged: (value) {
                                                setState(() {
                                                  selectedShipping = data;
                                                });
                                                processAddress(
                                                    selectedShipping!);
                                              }),
                                          Expanded(
                                              child: Container(
                                            decoration: BoxDecoration(
                                                border: Border.all(
                                                    width: 1,
                                                    color: Color(0xffE6E6E6)),
                                                borderRadius:
                                                    BorderRadius.circular(4)),
                                            child: Padding(
                                              padding:
                                                  const EdgeInsets.all(16.0),
                                              child: Row(
                                                children: [
                                                  SvgPicture.asset(
                                                      "$kIconsPath/logistic.svg"),
                                                  Text("${data.name}")
                                                ],
                                              ),
                                            ),
                                          ))
                                        ],
                                      ),
                                    );
                                  });
                            } else if (snapshot.hasError) {
                              return Text("Error");
                            } else {
                              return Text("Loading...");
                            }
                          }),
                    ),
                  ],
                ),
              ),
              Visibility(
                visible: isLogisticAvailable == "available",
                child: Container(
                  margin: EdgeInsets.symmetric(horizontal: 25),
                  child: Row(
                    children: [
                      Icon(Icons.access_time_outlined),
                      SizedBox(
                        width: 5,
                      ),
                      RichText(
                        text: TextSpan(
                          text: 'Delivery duration : ',
                          style: Styles.h3(
                              fontSize: 12, fontWeight: FontWeight.w500),
                          children: <TextSpan>[
                            TextSpan(
                                text:
                                    ' ${selectedShippingRegion?.durationMin ?? "-"} - ${selectedShippingRegion?.durationMax ?? "-"} Working days, ',
                                style: Styles.h3(
                                    fontSize: 12, fontWeight: FontWeight.w500)),
                            TextSpan(
                                text:
                                    'Price : ${selectedShippingRegion?.baseCost ?? "-"}'),
                          ],
                        ),
                      )
                    ],
                  ),
                ),
              ),
              Visibility(
                visible: isLogisticAvailable == "notAvailable",
                child: Container(
                  margin: EdgeInsets.symmetric(horizontal: 25, vertical: 10),
                  child: Row(
                    children: [
                      Icon(
                        Icons.access_time_outlined,
                        color: Colors.red,
                      ),
                      SizedBox(
                        width: 5,
                      ),
                      Expanded(
                          child: Text(
                              "The Logistics service you have selected does not deliver to your location.")),
                    ],
                  ),
                ),
              )
            ],
          ),
        ),
        Visibility(
          visible: deliveryMethod == "doorstep",
          child: Column(
            children: [
              SizedBox(
                height: 20,
              ),
              Container(
                margin: EdgeInsets.symmetric(horizontal: 25),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Expanded(
                        child: CustomTextFormField(
                      hintText: 'Enter Delivery Instructions',
                    )),
                  ],
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }

  Widget pickUp(BuildContext context) {
    return Column(
      children: [
        Visibility(
          visible: deliveryMethod == "pickup",
          child: Column(
            children: [
              SizedBox(
                height: 25,
              ),
              Container(
                margin: EdgeInsets.symmetric(horizontal: 25),
                child: Column(
                  children: [
                    Row(
                      children: [
                        Text("Pick Up Location"),
                      ],
                    ),
                    Container(
                      child: FutureBuilder<List<PickupLocation>>(
                          future: StoreBountyInit.getPickupLocations(),
                          builder: (context,
                              AsyncSnapshot<List<PickupLocation>> snapshot) {
                            if (snapshot.hasData) {
                              return ListView.builder(
                                  itemCount: snapshot.data!.length,
                                  shrinkWrap: true,
                                  padding: EdgeInsets.zero,
                                  physics: const NeverScrollableScrollPhysics(),
                                  itemBuilder: (context, index) {
                                    final data = snapshot.data![index];
                                    return Container(
                                      margin:
                                          EdgeInsets.symmetric(vertical: 10),
                                      child: Row(
                                        children: [
                                          Radio(
                                              value: data.id,
                                              activeColor: defaultColor ??
                                                  AppColors.black,
                                              groupValue: selectedPickup?.id,
                                              onChanged: (value) {
                                                setState(() {
                                                  selectedPickup = data;
                                                });

                                                processPickup(data);
                                              }),
                                          Expanded(
                                              child: Container(
                                            decoration: BoxDecoration(
                                                border: Border.all(
                                                    width: 1,
                                                    color: Color(0xffE6E6E6)),
                                                borderRadius:
                                                    BorderRadius.circular(4)),
                                            child: Padding(
                                              padding:
                                                  const EdgeInsets.all(16.0),
                                              child: Row(
                                                children: [
                                                  Icon(
                                                    Icons.location_on,
                                                    color: AppColors.black
                                                        .withOpacity(0.2),
                                                  ),
                                                  SizedBox(
                                                    width: 10,
                                                  ),
                                                  Expanded(
                                                    child: Column(
                                                      crossAxisAlignment:
                                                          CrossAxisAlignment
                                                              .start,
                                                      children: [
                                                        Text("${data.name}",
                                                            style: Styles.p3(
                                                                fontSize: 12,
                                                                fontWeight:
                                                                    FontWeight
                                                                        .w300)),
                                                        Row(
                                                          children: [
                                                            Expanded(
                                                                child: Text(
                                                              "${data.street1}",
                                                              style: Styles.p3(
                                                                  fontSize: 10,
                                                                  fontWeight:
                                                                      FontWeight
                                                                          .w300),
                                                            )),
                                                          ],
                                                        ),
                                                      ],
                                                    ),
                                                  )
                                                ],
                                              ),
                                            ),
                                          ))
                                        ],
                                      ),
                                    );
                                  });
                            } else if (snapshot.hasError) {
                              return Text("Error");
                            } else {
                              return Text("Loading...");
                            }
                          }),
                    )
                  ],
                ),
              )
            ],
          ),
        ),
      ],
    );
  }

  processDeliveryFee(snapshot) {
    if (deliveryMethod == "pickup") {
      deliveryFee = 0.0;
      return 0;
    }
    if (selectedShippingRegion != null) {
      print("process delivery ${snapshot.data['delivery']}");
      if (snapshot.data['delivery'] <= 1) {
        deliveryFee = selectedShippingRegion?.baseCost ?? 0;
        return selectedShippingRegion?.baseCost ?? 0;
      } else {
        var data = selectedShippingRegion?.baseCost +
            ((selectedShippingRegion?.costPerMetric) *
                (snapshot.data['delivery'] - 1));
        print("process delivery gg ${snapshot.data['deliveryQty']}");
        deliveryFee = data;
        return data;
      }
    } else {
      deliveryFee = 0.0;
      return 0;
    }
  }

  totalAmount(snapshot) {
    var data = snapshot.data['total'] +
        processDeliveryFee(snapshot) +
        processPaymentOptions();

    totalAmountMain = data;

    return data;
  }

  processPaymentOptions() {
    if (paymentMode == "online") {
      if (selectedPaymentGameWay != null) {
        var gateWayPercent = selectedPaymentGameWay!.percent;
        var surcharge = selectedPaymentGameWay!.surcharge;
        var passCharges = selectedPaymentGameWay!.passCharges;
        var cappedAt = selectedPaymentGameWay!.cappedAt;

        if (passCharges == true) {
          var gateFeeMain = totalAmountMain * (gateWayPercent! / 100);

          if (gateFeeMain > cappedAt) {
            return cappedAt;
          } else {
            return gateFeeMain;
          }
        } else {
          return 0;
        }
      } else {
        return 0;
      }
    } else {
      return 0;
    }
  }
}
