library storebounty_checkout_v1;

import 'dart:convert';
import 'dart:io';

import 'package:intl/intl.dart';
import 'package:storebounty_init/api/httpclient.dart';
import 'package:storebounty_init/storebountyinit.dart';

export 'src/entrypoint.dart';


class StorebountyCheckout {


  static String getCurrencySymbol(String? amount) {
    print(amount);
    var format =
    NumberFormat.simpleCurrency(locale: Platform.localeName, name: amount);
    return format.currencySymbol;
  }

  static Future getAddresses() async {
    try {

      final jsonData = await StoreBountyInit.readJsonFromRoot();

      final response =
      await HttpClientClass.getRequestWithAuthWithToken("${jsonData['BASE_URL']}/marketplace/v1/channel/stores/${jsonData['STORE_ID']}/customers/addresses");
      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future getLogistic() async {
    try {

      final jsonData = await StoreBountyInit.readJsonFromRoot();

      final response =
      await HttpClientClass.getRequestWithAuthWithToken("${jsonData['BASE_URL']}/accounts/api/v1/omolanke/${jsonData['STORE_ID']}/shippings");
      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future getPickupStations() async {
    try {

      final storeId = await StoreBountyInit.getStoreId();

      final response =
      await HttpClientClass.getRequestWithAuthWithToken("https://apis-dev.ibunchng.com/marketplace/api/v1/stores/$storeId/warehouses");
      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future getLocationData() async {
    try {

      final storeId = await StoreBountyInit.getStoreId();

      final response =
      await HttpClientClass.getRequestWithAuthWithToken("https://apis-dev.ibunchng.com/accounts/api/v1/public/countries");
      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future createAddress(body) async {
    try {

      final storeId = await StoreBountyInit.getStoreId();

      final response =
      await HttpClientClass.postRequestAuthOrder("https://apis-dev.ibunchng.com/marketplace/v1/channel/stores/$storeId/customers/addresses", body);
      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future createOrder(body) async {
    try {

      final storeId = await StoreBountyInit.getStoreId();

      final response =
      await HttpClientClass.postRequestAuthOrder("https://apis-dev.ibunchng.com/marketplace/v1/mobile/$storeId/orders/create", body);
      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future getReferenceNo(body) async {
    try {

      final storeId = await StoreBountyInit.getStoreId();

      final response =
      await HttpClientClass.postRequestAuthOrder("https://apis-dev.ibunchng.com/marketplace/v1/mobile/$storeId/payments/initialize", body);

      print(response.body);
      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

  static Future verifyTransaction(body) async {
    try {

      final storeId = await StoreBountyInit.getStoreId();

      final response =
      await HttpClientClass.postRequestAuthOrder("https://apis-dev.ibunchng.com/marketplace/v1/mobile/$storeId/payments/status", body);

      if (response.statusCode == 200) {
        return response;
      } else if (response.statusCode == 412) {
        var error = jsonDecode(response.body);
        throw (error['message']);
      } else {
        throw ("Unknown error");
      }
    } catch (e) {
      rethrow;
    }
  }

}
