import 'package:flutter/material.dart';
import 'package:storebounty_checkout/widget/styles/colors.dart';
import 'package:storebounty_checkout/widget/styles/textstles.dart';

class CustomDropDown extends StatelessWidget {
  final String hintText;
  final String? labelText;
  final Function(dynamic)? onChanged;
  final String? value;
  final List<Map<String, String>> options;
  const CustomDropDown({super.key, this.value = "", required this.options, required this.hintText, this.labelText, this.onChanged});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Visibility(
          visible: labelText != null,
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Text(
                labelText ?? "",
                style: Styles.p0(fontSize: 16, fontWeight: FontWeight.w500),
              ),
              SizedBox(height: 2),
            ],
          ),
        ),
        Container(
          padding: const EdgeInsets.only(left: 16, right: 7),
          decoration: BoxDecoration(
                border: Border.all(
                  color:Color(0xff97928c).withOpacity(0.5),
                  width: 0.5,
                ),
                borderRadius: const BorderRadius.all(
                  Radius.circular(4),
                ),
              ),
              // height: 40,
          child: Padding(
            padding: const EdgeInsets.symmetric(horizontal: 4, vertical: 10),
            child: DropdownButton(
              isExpanded: true,
              isDense: true,
                  underline: Container(),
                  elevation: 0,
              value: value,
              hint: Text(hintText, style: Styles.p3(fontSize: 12, color: AppColors.black.withOpacity(0.5))),
              icon: const Icon(Icons.arrow_drop_down),
              onChanged: onChanged,
              items: options
                  .map((valueMain) {
                return DropdownMenuItem(
                  value: valueMain['value'],
                  child: Text(valueMain['name']!, style: Styles.p3(fontSize: 16),),
                );
              }).toList(),
            ),
          ),
        ),
      ],
    );
  }
}